#include <config.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/accelmap.h>
#include "utility.h"
#include "Application.h"
#include "DocumentSystem.h"
#include "ActionSystem.h"

#include "actions/Selection.h"
#include "actions/DeleteSubtitleSelect.h"
#include "actions/AdjustTime.h"
#include "actions/InsertSubtitle.h"
#include "actions/NewDocument.h"
#include "actions/OpenDocument.h"
#include "actions/OpenTranslation.h"
#include "actions/ApplyTranslation.h"
#include "actions/ExternalVideoPlayer.h"

#include "gui/AboutUI.h"
#include "gui/ScaleUI.h"
#include "gui/ChangeFramerateUI.h"
#include "gui/MoveSubtitlesUI.h"
#include "gui/OpenDocumentUI.h"
#include "gui/SaveDocumentUI.h"
#include "gui/SpellCheckingUI.h"
#include "gui/FindAndReplaceUI.h"
#include "gui/StyleEditorUI.h"
#include "gui/SplitDocumentUI.h"
#include "gui/JoinDocumentUI.h"
#include "gui/CheckErrorsUI.h"
#include "gui/OpenMovieUI.h"

#include "formats/SubtitleText.h"

#include "SubtitleSystem.h"

/*
 *
 */
Application::Application(BaseObjectType *cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)//int argc, char *argv[])
:Gtk::Window(cobject)
{
	refGlade->get_widget_derived("statusbar", m_statusbar);
	m_menubar.create(*this, *m_statusbar);

	refGlade->get_widget("vbox-main", m_vboxMain);
	refGlade->get_widget_derived("video-player", m_videoPlayer);
	refGlade->get_widget_derived("waveform-system", m_waveform_system);
	refGlade->get_widget("notebook-documents", m_notebook_documents);

	//m_notebook_documents->set_scrollable(true);

	set_default_size(800,600);

	m_vboxMain->pack_start(m_menubar, false, false);
	m_vboxMain->reorder_child(m_menubar, 0);

	m_menubar.show_all();

	DocumentSystem::getInstance().signal_document_create().connect(
			sigc::mem_fun(*this, &Application::on_document_create));

	DocumentSystem::getInstance().signal_document_delete().connect(
			sigc::mem_fun(*this, &Application::on_document_delete));

	m_notebook_documents->signal_switch_page().connect(
			sigc::mem_fun(*this, &Application::on_signal_switch_page));

	// on va chercher la configuration clavier
	Glib::ustring path_se_accelmap = get_config_dir("accelmap");
	Gtk::AccelMap::load(path_se_accelmap);

	m_waveform_system->set_video_player(m_videoPlayer->get_video_player());

	ActionSystem::getInstance().signal_emit().connect(
			sigc::mem_fun(*this, &Application::on_execute_action));


	//
	Config::getInstance().signal_changed("interface").connect(
			sigc::mem_fun(*this, &Application::on_config_interface_changed));

	show();

	load_config();
}

/*
 *
 */
Application::~Application()
{
	se_debug(SE_DEBUG_APP);

	m_autosave_timeout.disconnect();

	Glib::ustring path_se_accelmap = get_config_dir("accelmap");
	Gtk::AccelMap::save(path_se_accelmap);
}

/*
 *
 */
void Application::load_config()
{
	//
	// Config
	//
	Config &cfg = Config::getInstance();

	// dynamic keyboar shorcuts
	bool value;
	cfg.get_value_bool("interface", "use-dynamic-keyboard-shortcuts", value, true);
	Gtk::Settings::get_default()->property_gtk_can_change_accels() = value;

	// maximize window
	cfg.get_value_bool("interface", "maximize-window", value, false);
	if(value)
		maximize();

	cfg.get_value_bool("interface", "display-video-player", value, false);
	set_display_video_player(value);

	cfg.get_value_bool("interface", "display-waveform", value, false);
	set_display_waveform(value);

	// first launch
	if(!cfg.has_group("encodings"))
	{
		cfg.set_value_string("encodings", "encodings", "ISO-8859-15;UTF-8");
		cfg.set_value_bool("encodings", "used-auto-detected", true);
	}

	cfg.get_value_bool("interface", "used-autosave", value, false);
	if( value )
	{
		int autosave_minutes = 0;
		cfg.get_value_int("interface", "autosave-minutes", autosave_minutes, 10);
		
		SubtitleTime time(0, autosave_minutes, 0, 0);

		m_autosave_timeout = Glib::signal_timeout().connect(
				sigc::mem_fun(*this, &Application::on_autosave_files), time.totalmsecs);
	}
}

/*
 *
 */
bool Application::on_delete_event(GdkEventAny *ev)
{
	bool res = Gtk::Window::on_delete_event(ev);

	Glib::ustring path_se_accelmap = get_config_dir("accelmap");
	Gtk::AccelMap::save(path_se_accelmap);

	return ask_to_save_on_exit();
	
	return res;
}

/*
 *	il y a la création d'un nouveau document
 *	on l'ajoute dans le notebook
 *	signal emit par DocumentSystem::signal_document_create
 */
void Application::on_document_create(Document *doc)
{
	g_return_if_fail(doc);

	Glib::ustring filename = doc->getName();

	//
	Gtk::HBox *hbox = NULL;
	Gtk::Image *image = NULL;
	Gtk::Button *close_button = NULL;
	Gtk::EventBox *eventbox = NULL;
	Gtk::Label* label = NULL;

	// hbox
	hbox = manage(new Gtk::HBox(false, 4));
	hbox->set_border_width(0);

	// event box
	eventbox = manage(new Gtk::EventBox);
	eventbox->set_visible_window(false);
	hbox->pack_start(*eventbox, true, true, 0);

	// label
	label = manage(new Gtk::Label(filename));
	label->set_use_markup(true);
	label->set_alignment(0.0, 0.5);
	label->set_padding(0, 0);
	eventbox->add(*label);

	// close button
	close_button = manage(new Gtk::Button);
	close_button->set_relief(Gtk::RELIEF_NONE);
	close_button->set_focus_on_click(false);
	close_button->set_border_width(0);

	close_button->signal_clicked().connect(
			sigc::bind<Document*>(
				sigc::mem_fun(*this, &Application::on_close_document), doc));

	// rcstyle for the button
	close_button->get_modifier_style()->set_xthickness(0);
	close_button->get_modifier_style()->set_ythickness(0);
	
	// close image 
	image = manage(new Gtk::Image(Gtk::Stock::CLOSE, Gtk::ICON_SIZE_MENU));
	close_button->add(*image);
	hbox->pack_start(*close_button, false, false, 0);

	// show all	
	hbox->show_all();


	//
	int id = 0;
	Gtk::ScrolledWindow *scroll = NULL;
	Gtk::Tooltips *tooltips = NULL;
	Gtk::Widget *page = NULL;


	scroll = manage(new Gtk::ScrolledWindow);
	scroll->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	scroll->add(*doc->get_subtitle_view());
	scroll->show();
	

	id = m_notebook_documents->append_page(*scroll, *hbox);


	tooltips = manage(new Gtk::Tooltips);
	m_notebook_documents->set_current_page(id);


	page = m_notebook_documents->get_nth_page(id);

	page->set_data("event", eventbox);
	page->set_data("document", doc);
	page->set_data("label", label);
	page->set_data("tooltips", tooltips);

	doc->get_signal("document-property-changed").connect(
			sigc::bind<Document*>(
				sigc::mem_fun(*this, &Application::update_document_property), doc));


	update_document_property(doc);

	DocumentSystem::getInstance().setCurrentDocument(doc);

	connect_document(doc);
}

/*
 *	retourne le widget (notebook) par rapport au document
 */
Gtk::Widget* Application::get_widget(Document *doc)
{
	for( int i = 0; i < m_notebook_documents->get_n_pages(); ++i)
	{
		Gtk::Widget *w = m_notebook_documents->get_nth_page(i);

		Document *document = (Document*)w->get_data("document");

		if(document == doc)
			return w;
	}
	return NULL;
}

/*
 *	mise a jour d'info (filename, tooltip, ...) dans la page
 */
void Application::update_document_property(Document *doc)
{
	Gtk::Widget *widget = get_widget(doc);
	g_return_if_fail(widget);

	Gtk::Widget *event = (Gtk::Widget*)widget->get_data("event");
	g_return_if_fail(event);

	Gtk::Label *label = (Gtk::Label*)widget->get_data("label");
	g_return_if_fail(label);

	Gtk::Tooltips *tooltips = (Gtk::Tooltips*)widget->get_data("tooltips");
	g_return_if_fail(tooltips);

	// mise à jour du nom du document
	label->set_text(doc->getName());

	// mise à jour du tooltip
	Glib::ustring tip = build_message("%s %s\n\n%s %s\n%s %s\n%s %s",
			_("Name:"), doc->getFilename().c_str(),
			_("Encoding:"), doc->getCharset().c_str(),
			_("Format:"), doc->getFormat().c_str(),
			_("NewLine:"), doc->getNewLine().c_str());

	tooltips->force_window();
	label->set_use_markup(true);
	tooltips->set_tip(*event, tip);
	label->set_use_markup(true);

}


/*
 *	on efface le document du notebook
 *	signal emit par DocumentSystem::signal_document_delete
 */
void Application::on_document_delete(Document *doc)
{
	if(doc == NULL)
		return;

	Gtk::Widget *widget = get_widget(doc);

	if(widget != NULL)
		m_notebook_documents->remove_page(*widget);
}

/*
 *
 */
void Application::on_current_document_changed(Document *doc)
{
	/*
	int page = -1;

	for( int i = 0; i < m_notebook_documents->get_n_pages(); ++i)
	{
		Gtk::Widget *w = m_notebook_documents->get_nth_page(i);

		Document *document = (Document*)w->get_data("document");

		if(document == doc)
		{
			m_notebook_documents->;
			return;
		}
	}
	*/
}

/*
 *
 */
void Application::on_close_document(Document *doc)
{
	g_return_if_fail(doc);

	DocumentSystem::getInstance().remove(doc);
}

/*
 *	Changement dans le notebook de la page editer
 *	On recupere la page pusi on init DocumentManager avec le document
 */
void Application::on_signal_switch_page(GtkNotebookPage* page, guint page_num)
{
	Gtk::Widget *w = m_notebook_documents->get_nth_page(page_num);
	
	if(w)
	{
		Document *doc = (Document*)w->get_data("document");

		if(doc)
		{
			DocumentSystem::getInstance().setCurrentDocument(doc);
			connect_document(doc);
		}
		else
		{
			DocumentSystem::getInstance().setCurrentDocument(NULL);
			disconnect_document(NULL);
		}
	}
}

/*
 *
 */
void Application::connect_document(Document *doc)
{
	disconnect_document(doc);

	if(doc)
	{
		std::cout << "connect document: " << doc->getName() << std::endl;

		// connect document message
		m_document_connections.push_back(
				doc->get_signal_message().connect(
					sigc::mem_fun(m_statusbar, &Statusbar::push_text)));
	
		m_document_connections.push_back(
				doc->get_signal_flash_message().connect(
					sigc::mem_fun(m_statusbar, &Statusbar::flash_message)));
	}
}

/*
 *
 */
void Application::disconnect_document(Document *doc)
{
	// clear old connection
	std::list<sigc::connection>::iterator it;
	for(it = m_document_connections.begin(); it!= m_document_connections.end(); ++it)
		(*it).disconnect();
	m_document_connections.clear();

	if(doc)
		std::cout << "disconnect document: " << doc->getName() << std::endl;
}

/*
 *
 */
void Application::on_execute_action(const Glib::ustring &name)
{
//#define REGISTER(name, callback) m_callback[name].connect(sigc::mem_fun(*this, &MenuBar::callback));
#define REGISTER(action, callback) if(action == name) { callback(); return; }

	// FILE
	REGISTER("new-subtitle", new_subtitle);
	REGISTER("open-subtitle", open_subtitle);
	REGISTER("save-subtitle", save_subtitle);
	REGISTER("save-as-subtitle", save_as_subtitle);

	REGISTER("close-subtitle", close_subtitle);
	REGISTER("quit", quit);

	REGISTER("import-text", import_text);
	REGISTER("export-text", export_text);
	REGISTER("open-translation", open_translation);
	REGISTER("save-translation", save_translation);

	// EDIT
	REGISTER("select-previous-subtitle", select_previous_subtitle);
	REGISTER("select-next-subtitle", select_next_subtitle);
	
	REGISTER("delete-subtitle", delete_subtitle);

	REGISTER("insert-before", insert_before);
	REGISTER("insert-after", insert_after);

	REGISTER("move-subtitles", move_subtitles);

	REGISTER("add-100-ms", add_msecs);
	REGISTER("remove-100-ms", remove_msecs);
	REGISTER("add-100-ms-to-start", add_msecs_to_start);
	REGISTER("remove-100-ms-to-start", remove_msecs_to_start);
	REGISTER("add-100-ms-to-duration", add_msecs_to_duration);
	REGISTER("remove-100-ms-to-duration", remove_msecs_to_duration);

	REGISTER("apply-translation", apply_translation);

	// TOOLS
	REGISTER("find-and-replace", find_and_replace);
	REGISTER("check-errors", check_errors);
	REGISTER("spell-check", spell_check);
	REGISTER("style-editor", style_editor);
	REGISTER("split-document", split_document);
	REGISTER("join-document", join_document);
	REGISTER("change-framerate", change_framerate);
	REGISTER("scale", scale);

	// ANOTHER
	REGISTER("preview/open", open_movie_for_external_video_player);
	REGISTER("preview/play", play_movie_with_external_video_player);
/*
	// VIEW
	REGISTER("setup-view-simple", setup_view_simple);
	REGISTER("setup-view-advanced", setup_view_advanced);
	REGISTER("setup-view-translation", setup_view_translation);
	REGISTER("setup-view-timing", setup_view_timing);
*/
	//
	REGISTER("about", about);

#undef REGISTER
}

/*
 *
 */
void Application::new_subtitle()
{
	// crée un nom valide
	Glib::ustring name("Untitled-");
	unsigned int n = 1;

	DocumentSystem &ds = DocumentSystem::getInstance();

	while( ds.getDocument(name + to_string(n) ) != NULL)
	{
		++n;
	}

	Glib::ustring final_name = name + to_string(n);
	// crée le document
	NewDocument action;
	if(action.execute(final_name))
		;//ds.getDocument(final_name));

	ds.setCurrentDocument(ds.getDocument(final_name));

}

/*
 *
 */
void Application::open_subtitle()
{
	static OpenDocumentUI ui;

	ui.show();
	if(ui.run() == Gtk::RESPONSE_OK)
	{
		Glib::ustring encoding = ui.getEncoding();

		std::list<Glib::ustring> filenames = ui.get_filenames();

		for(std::list<Glib::ustring>::const_iterator it=filenames.begin();
				it != filenames.end(); ++it)
		{
			// verifie qu'il n'est pas déjà ouvert
			if(DocumentSystem::getInstance().getDocument(*it) == NULL)
			{
				OpenDocument().execute(*it, encoding);
			}
		}
	}
	ui.hide();
}

/*
 *
 */
void Application::open_translation()
{
	static OpenDocumentUI ui;

	ui.show();
	if(ui.run() == Gtk::RESPONSE_OK)
	{
		Glib::ustring encoding = ui.getEncoding();
		Glib::ustring filename = ui.get_filename();
			
		if(OpenTranslation().execute(filename, encoding))
			ActionSystem::getInstance().execute("view/translation");
	}
	ui.hide();
}

/*
 *
 */
void Application::save_translation()
{
	Document *current = DocumentSystem::getInstance().getCurrentDocument();

	if(current == NULL)
		return;

	static SaveDocumentUI ui;

	ui.show();
	if(ui.run() == Gtk::RESPONSE_OK)
	{
		Glib::ustring filename = ui.get_filename();
		Glib::ustring format = ui.getFormat();
		Glib::ustring encoding = ui.getEncoding();
		Glib::ustring newline = ui.getNewLine();

		try
		{
			Document doc_translation(*current);

			doc_translation.setFormat(filename);
			doc_translation.setFormat(format);
			doc_translation.setCharset(encoding);
			doc_translation.setNewLine(newline);

			// apply translation
			Gtk::TreeIter it = doc_translation.get_subtitle_model()->getFirst();
			while(it)
			{
				SubtitleModifier sub(it);
				sub.set_text(sub.get_translation());

				++it;
			}
			//
			if(doc_translation.save(filename))
				current->flash_message(_("Saving translation file %s (%s, %s, %s)."), 
						filename.c_str(), format.c_str(), encoding.c_str(), newline.c_str());
			else
				current->message(_("The translation file %s (%s, %s, %s) has not been saved."), 
						filename.c_str(), format.c_str(), encoding.c_str(), newline.c_str()); 
		}
		catch(...)
		{
		}
	}

	ui.hide();
}

/*
 *
 */
void Application::save_subtitle()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc == NULL)
		return;

	if(Glib::file_test(doc->getFilename(), Glib::FILE_TEST_EXISTS))
	{
		Glib::ustring filename = doc->getFilename();
		Glib::ustring format = doc->getFormat();
		Glib::ustring charset = doc->getCharset();
		Glib::ustring newline = doc->getNewLine();

		if(doc->save(filename))
			doc->flash_message(_("Saving file %s (%s, %s, %s)."), 
						filename.c_str(), format.c_str(), charset.c_str(), newline.c_str());
		else
			doc->message(_("The file %s (%s, %s, %s) has not been saved."), 
						filename.c_str(), format.c_str(), charset.c_str(), newline.c_str()); 
	}
	else
		save_as_subtitle();
}

/*
 *
 */
void Application::save_as_subtitle()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc == NULL)
		return;

	static SaveDocumentUI ui;

	ui.show();
	if(ui.run() == Gtk::RESPONSE_OK)
	{
		Glib::ustring filename = ui.get_filename();
		Glib::ustring format = ui.getFormat();
		Glib::ustring encoding = ui.getEncoding();
		Glib::ustring newline = ui.getNewLine();

		Document *doc = DocumentSystem::getInstance().getCurrentDocument();

		if(doc != NULL)
		{
			doc->setFormat(format);
			doc->setCharset(encoding);
			doc->setNewLine(newline);

			if(doc->save(filename))
				doc->flash_message(_("Saving file %s (%s, %s, %s)."), 
						filename.c_str(), format.c_str(), encoding.c_str(), newline.c_str());
			else
				doc->message(_("The file %s (%s, %s, %s) has not been saved."), 
						filename.c_str(), format.c_str(), encoding.c_str(), newline.c_str()); 
		}
	}

	ui.hide();
}

/*
 *
 */
void Application::import_text()
{
#warning "TODO rewriting that!"

	DialogFileChooser ui(_("Import Text"), "dialog-import-text", 
			Gtk::FILE_CHOOSER_ACTION_OPEN, EXTRA_ENCODING_WITH_AUTO_DETECTED);

	if(ui.run() == Gtk::RESPONSE_OK)
	{
		Glib::ustring filename = ui.get_filename();
		Glib::ustring charset = ui.getEncoding();

		try
		{
			Document *doc = new Document();
			doc->setCharset(charset);

			SubtitleText reader(doc);
			if(reader.open(filename))
			{
				doc->setFilename(DocumentSystem::getInstance().generate_untitled_name());
				
				DocumentSystem::getInstance().append(doc);
			}
			else
			{
				delete doc;
			}
		}
		catch(const Glib::Error &ex)
		{
			std::cerr << ex.what() << std::endl;
		}
	}
}

/*
 *
 */
void Application::export_text()
{
#warning "TODO rewriting that!"

	DialogFileChooser ui(_("Export Text"), "dialog-export-text", 
			Gtk::FILE_CHOOSER_ACTION_OPEN, EXTRA_ENCODING | EXTRA_NEWLINE);

	if(ui.run() == Gtk::RESPONSE_OK)
	{
		Glib::ustring filename = ui.get_filename();
		Glib::ustring charset = ui.getEncoding();
		Glib::ustring newline = ui.getNewLine();

		try
		{
			Document *doc = DocumentSystem::getInstance().getCurrentDocument();

			if(doc != NULL)
			{
				Document copy(*doc);
				copy.setCharset(charset);
				copy.setNewLine(newline);
				copy.setFilename(filename);

				SubtitleText w(&copy);
				if(w.save(filename))
				{
				}
				else
					;//error
			}
		}
		catch(...)
		{
		}
	}
}


/*
 *
 */
void Application::close_subtitle()
{
	DocumentSystem &ds = DocumentSystem::getInstance();

	Document *doc = ds.getCurrentDocument();

	if(doc)
	{
		ds.remove(doc);
	}
}

/*
 *
 */
void Application::quit()
{
	Gtk::Main::quit();
}

// EDIT

/*
 *
 */
void Application::select_previous_subtitle() 
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc)
	{
		Selection action;
		action.execute(*doc, Selection::PREVIOUS);
	}
}

/*
 *
 */
void Application::select_next_subtitle()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc)
	{
		Selection action;
		action.execute(*doc, Selection::NEXT);
	}
}

/*
 *
 */
void Application::delete_subtitle()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc)
	{
		DeleteSubtitleSelect action;
		action.execute(*doc);
	}
}

/*
 *
 */
void Application::insert_before()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc)
	{
		InsertSubtitle action;
		action.execute(*doc, InsertSubtitle::BEFORE);
	}
}

/*
 *
 */
void Application::insert_after()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc)
	{
		InsertSubtitle action;
		action.execute(*doc, InsertSubtitle::AFTER);
	}
}

/*
 *
 */
void Application::move_subtitles()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc)
	{
		MoveSubtitlesUI action;
		action.execute();
	}
}

/*
 *
 */
void Application::add_msecs()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc)
	{
		AdjustTime action;
		action.execute(*doc, 100, true, true);
	}
}

/*
 *
 */
void Application::remove_msecs()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc)
	{
		AdjustTime action;
		action.execute(*doc, -100, true, true);
	}
}

/*
 *
 */
void Application::add_msecs_to_start()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc)
	{
		AdjustTime action;
		action.execute(*doc, 100, true, false);
	}
}

/*
 *
 */
void Application::remove_msecs_to_start()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc)
	{
		AdjustTime action;
		action.execute(*doc, -100, true, false);
	}
}

/*
 *
 */
void Application::add_msecs_to_duration()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc)
	{
		AdjustTime action;
		action.execute(*doc, 100, false, true);
	}
}

/*
 *
 */
void Application::remove_msecs_to_duration()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc)
	{
		AdjustTime action;
		action.execute(*doc, -100, false, true);
	}
}

/*
 *
 */
void Application::apply_translation()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc)
	{
		ApplyTranslation action;
		action.execute(*doc);
	}
}

// TOOLS

/*
 *
 */
void Application::find_and_replace()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc)
	{
		FindAndReplaceUI action;
		action.execute();
	}
}

/*
 *
 */
void Application::check_errors()
{
	createDialogCheckErrors();
}

/*
 *
 */
void Application::spell_check()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc)
	{
		SpellCheckingUI action;
		action.execute();
	}
}

/*
 *
 */
void Application::style_editor()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc)
	{
		StyleEditorUI action;
		action.execute();
	}
}

/*
 *
 */
void Application::split_document()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc)
	{
		SplitDocumentUI action;
		action.execute();
	}
}

/*
 *
 */
void Application::join_document()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc)
	{
		JoinDocumentUI action;
		action.execute();
	}
}

/*
 *
 */
void Application::change_framerate()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc)
	{
		ChangeFramerateUI action;
		action.execute();
	}
}

/*
 *
 */
void Application::scale()
{
	Document *doc = DocumentSystem::getInstance().getCurrentDocument();

	if(doc)
	{
		ScaleUI action;
		action.execute();
	}
}

/*
 *
 */
void Application::about()
{
	AboutUI ui;
	ui.run();
}

/*
 *
 */
void Application::on_config_interface_changed(const Glib::ustring &key, const Glib::ustring &value)
{
	if(key == "use-dynamic-keyboard-shortcuts")
	{
		bool state;
		from_string(value, state);

	  Gtk::Settings::get_default()->property_gtk_can_change_accels() = state;
	}
	else if(key == "maximize-window")
	{
		bool state;
		from_string(value, state);

		if(state)
			maximize();
		else 
			unmaximize();
	}
	else if(key == "display-video-player")
	{
		bool state;
		from_string(value, state);

		set_display_video_player(state);
	}
	else if(key == "display-waveform")
	{
		bool state;
		from_string(value, state);

		set_display_waveform(state);
	}
	else if(key == "used-autosave")
	{
		if(m_autosave_timeout)
			m_autosave_timeout.disconnect();

		bool state;
		from_string(value, state);

		if(state)
		{
			if(m_autosave_timeout)
				m_autosave_timeout.disconnect();

			int autosave_minutes = 0;
			Config::getInstance().get_value_int("interface", "autosave-minutes", autosave_minutes, 10);
			
			SubtitleTime time(0, autosave_minutes, 0, 0);

			m_autosave_timeout = Glib::signal_timeout().connect(
					sigc::mem_fun(*this, &Application::on_autosave_files), time.totalmsecs);
		}
	}
}

/*
 *
 */
bool Application::ask_to_save_on_exit()
{
	bool askToSaveOnExit = false;
	if(Config::getInstance().get_value_bool("interface", "ask-to-save-on-exit", askToSaveOnExit))
	{
		if(askToSaveOnExit)
		{
			Gtk::MessageDialog dialog(*this, 
					_("Do you want to save the subtitle before the exit?"),
					false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO, true);

			if(dialog.run() == Gtk::RESPONSE_YES)
			{
#warning "FIXME: Ask To Save On Exit"
				return true;
			}
		}
	}

	return false;
}

/*
 *
 */
void Application::set_display_video_player(bool state)
{
	Gtk::Paned *paned = dynamic_cast<Gtk::Paned*>(m_videoPlayer->get_parent());

	g_return_if_fail(paned);

	if(state)
	{
		m_videoPlayer->show();

		if(!paned->is_visible())
			paned->show();
	}
	else
	{
		m_videoPlayer->hide();

		if(paned->get_child1()->is_visible() == false && paned->get_child2()->is_visible() == false)
			paned->hide();
	}
}

/*
 *
 */
void Application::set_display_waveform(bool state)
{
	Gtk::Paned *paned = dynamic_cast<Gtk::Paned*>(m_waveform_system->get_parent());

	g_return_if_fail(paned);

	if(state)
	{
		m_waveform_system->show();

		if(!paned->is_visible())
			paned->show();
	}
	else
	{
		m_waveform_system->hide();

		if(paned->get_child1()->is_visible() == false && paned->get_child2()->is_visible() == false)
			paned->hide();
	}
}

// another
void Application::open_movie_for_external_video_player()
{
	OpenMovieUI ui;
	if(ui.run() == Gtk::RESPONSE_OK)
	{
		Glib::ustring uri = ui.get_uri();

		m_uri_movie_external_video_player = uri;
	}
}

void Application::play_movie_with_external_video_player()
{
	if(m_uri_movie_external_video_player.empty())
	{
		open_movie_for_external_video_player();
	}

	if(m_uri_movie_external_video_player.empty() == false)
	{
		ExternalVideoPlayer().execute(m_uri_movie_external_video_player);
	}
}

bool Application::on_autosave_files()
{
	DocumentList docs = DocumentSystem::getInstance().getAllDocuments();

	if(docs.size() == 0)
		return true;
	
	m_statusbar->flash_message("Autosave files...");

	DocumentList::iterator it;
	
	for(it = docs.begin(); it != docs.end(); ++it)
	{
		Glib::ustring filename = (*it)->getFilename();
		/*
		Glib::ustring format = (*it)->getFormat();
		Glib::ustring charset = (*it)->getCharset();
		*/
#warning "FIXME: if filename doesn't exist"
		(*it)->save(filename);
	}

	m_statusbar->flash_message("Autosave files...ok");
	return true;
}

void Application::init(OptionGroup &options)
{
	// files
	for(unsigned int i = 0; i< options.files.size(); ++i)
	{
		Glib::ustring filename = options.files[i];

		if(	Glib::file_test(filename, Glib::FILE_TEST_EXISTS | Glib::FILE_TEST_IS_REGULAR) && 
				Glib::file_test(filename, Glib::FILE_TEST_IS_DIR) == false)
		{
			Document *doc = new Document;
			doc->setFormat("");
			doc->setCharset(options.encoding);

			if(doc->open(filename))
			{
				DocumentSystem::getInstance().append(doc);
				DocumentSystem::getInstance().setCurrentDocument(doc);
			}
			else
			{
				delete doc;
			}
		}
	}

	// video
	if(!options.video.empty())
	{
		try
		{
			Glib::ustring uri = Glib::filename_to_uri(utility::create_full_path(options.video));

			m_videoPlayer->open(uri);
		}
		catch(const Glib::Error &ex)
		{
			std::cerr << ex.what() << std::endl;
		}
	}

	// waveform
	if(!options.waveform.empty())
	{
		try
		{
			Glib::ustring uri = Glib::filename_to_uri(utility::create_full_path(options.waveform));

			m_waveform_system->open(uri);
		}
		catch(const Glib::Error &ex)
		{
			std::cerr << ex.what() << std::endl;
		}
	}
}
