#include <libglademm/xml.h>
#include <gtkmm.h>

#include "utility.h"
#include "DocumentSystem.h"
#include "DialogUtility.h"
#include "CheckErrorsUI.h"

/*
 *
 */
CheckErrors::CheckErrors()
{
	m_minCPS = 5;
	m_maxCPS = 25;
	m_maxCPL = 40;
	m_minDisplay = 1000;
	m_minGBS = 100;
	m_maxLine = 2;

	load_config();
}

/*
 *
 */
CheckErrors::~CheckErrors()
{
}

/*
 *
 */
void CheckErrors::load_config()
{
	Config &cfg = Config::getInstance();

	cfg.get_value_int("timing", "min-characters-per-second", m_minCPS, m_minCPS);
	cfg.get_value_int("timing", "max-characters-per-second", m_maxCPS, m_maxCPS);
	cfg.get_value_int("timing", "max-characters-per-line", m_maxCPL, m_maxCPL);
	cfg.get_value_int("timing", "min-display", m_minDisplay, m_minDisplay);
	cfg.get_value_int("timing", "min-gap-between-subtitle", m_minGBS, m_minGBS);
	cfg.get_value_int("timing", "max-line", m_maxLine, m_maxLine);
}


/*
 *	chevauchement
 *	Une erreur est détectée lorsque deux sous-titres se chevauchent.
 */
bool CheckErrors::check_overlapping(const SubtitleModifier &sub, Glib::ustring &msg)
{
	Gtk::TreeIter iter_next = sub.get_iter();
	++iter_next;

	if(!iter_next)
		return false;

	SubtitleModifier next(iter_next);

	if(sub.get_end() > next.get_start())
	{
		msg = build_message(
						_("<b>Subtitle overlap on next subtitle:</b> %ims overlap\n%s -> %s\n%s"), 
						(sub.get_end() - next.get_start()).totalmsecs,
						sub.get_end().str().c_str(), next.get_start().str().c_str(),
						sub.get_text().c_str()); 
		return true;
	}
	return false;
}

/*
 *	temps trop court
 *	Une erreur est détectée lorsque le nombre de caractères par seconde
 *	est strictement supérieur à une valeur définie par l'utilisateur.
 */
bool CheckErrors::check_short_display_time(const SubtitleModifier &sub, Glib::ustring &msg)
{
	int cps = calcul_cps(sub);

	if(cps > m_maxCPS)
	{
		msg = build_message(
				_("<b>Subtitle display time is too short:</b> %i char/s\n%s -> %s\n%s"), 
						(int)cps,
						sub.get_start().str().c_str(), sub.get_end().str().c_str(),
						sub.get_text().c_str()); 
		return true;
	}
	return false;
}
	
/*
 *	temps trop long
 *	Une erreur est détectée lorsque le nombre de caractères par seconde
 *	est strictement inférieur à une valeur définie par l'utilisateur.
 */
bool CheckErrors::check_long_display_time(const SubtitleModifier &sub, Glib::ustring &msg)
{
	int cps = calcul_cps(sub);

	if(cps < m_minCPS)
	{
		msg = build_message(
				_("<b>Subtitle display time is too long:</b> %i char/s\n%s -> %s\n%s"), 
						(int)cps,
						sub.get_start().str().c_str(), sub.get_end().str().c_str(),
						sub.get_text().c_str()); 
		return true;
	}
	return false;
}

/*
 *	ligne trop longue
 *	Une erreur est détectée lorsque la longueur d'une ligne de sous-titre
 *	est strictement supérieur à une valeur spécifiée par l'utilisateur.
 */
bool CheckErrors::check_long_line_text(const SubtitleModifier &sub, Glib::ustring &msg)
{
	// >= m_maxCPL
	std::istringstream iss(sub.get_characters_per_line_text());
	
	std::string line;
	
	int val=0;

	while(std::getline(iss, line))
	{
		from_string(line, val);

		if(val > m_maxCPL)
		{
			msg = build_message(
						_("<b>Subtitle has a too long line (text):</b> %i characters\n%s -> %s\n%s"),
						val,
						sub.get_start().str().c_str(), sub.get_end().str().c_str(),
						sub.get_text().c_str()); 
			return true;
		}
	}

	return false;
}

/*
 *	ligne trop longue
 *	Une erreur est détectée lorsque la longueur d'une ligne de sous-titre
 *	est strictement supérieur à une valeur spécifiée par l'utilisateur.
 */
bool CheckErrors::check_long_line_translation(const SubtitleModifier &sub, Glib::ustring &msg)
{
	// >= m_maxCPL
	std::istringstream iss(sub.get_characters_per_line_translation());
	
	std::string line;
	
	int val=0;

	while(std::getline(iss, line))
	{
		from_string(line, val);

		if(val > m_maxCPL)
		{
			msg = build_message(
						_("<b>Subtitle has a too long line (translation):</b> %i characters\n%s -> %s\n%s"),
						val,
						sub.get_start().str().c_str(), sub.get_end().str().c_str(),
						sub.get_translation().c_str()); 
			return true;
		}
	}

	return false;
}

/*
 *	trop de ligne
 */
bool CheckErrors::check_max_line_text(const SubtitleModifier &sub, Glib::ustring &msg)
{
	// >= m_maxCPL
	std::istringstream iss(sub.get_characters_per_line_text());
	
	std::string line;
	
	int count=0;

	while(std::getline(iss, line))
	{
		++count;
	}

	if(count > m_maxLine)
	{
		msg = build_message(
					_("<b>Subtitle has too many lines (text):</b> %i\n%s -> %s\n%s"),
					count,
					sub.get_start().str().c_str(), sub.get_end().str().c_str(),
					sub.get_text().c_str()); 
		return true;
	}

	return false;
}

/*
 *	trop de ligne
 */
bool CheckErrors::check_max_line_translation(const SubtitleModifier &sub, Glib::ustring &msg)
{
	// >= m_maxCPL
	std::istringstream iss(sub.get_characters_per_line_translation());
	
	std::string line;
	
	int count=0;

	while(std::getline(iss, line))
	{
		++count;
	}

	if(count > m_maxLine)
	{
		msg = build_message(
					_("<b>Subtitle has too many lines (translation):</b> %i\n%s -> %s\n%s"),
					count,
					sub.get_start().str().c_str(), sub.get_end().str().c_str(),
					sub.get_translation().c_str()); 
		return true;
	}

	return false;
}

/*
 *	transition
 *	Respect du temps entre chaque s-t
 */
bool CheckErrors::check_minimum_gap_between_subtitle(const SubtitleModifier &sub, Glib::ustring &msg)
{
	Gtk::TreeIter iter_next = sub.get_iter();
	++iter_next;

	if(!iter_next)
		return false;

	SubtitleModifier next(iter_next);

	long res = (next.get_start() - sub.get_end()).totalmsecs;

	if(res < (int)m_minGBS)
	{
		msg = build_message(
					_("<b>Too short gap between subtitle:</b> %ims"), res);	
		return true;
	}
	return false;
}

/*
 *	affichage trop court d'un sous-titre
 */
bool CheckErrors::check_minimum_display_subtitle(const SubtitleModifier &sub, Glib::ustring &msg)
{
	if(sub.get_duration().totalmsecs < (int)m_minDisplay)
		return true;

	return false;
}

/*
 *	calcul le nombre de caractères par seconde
 */
int CheckErrors::calcul_cps(const SubtitleModifier &sub)
{
	unsigned int len = sub.get_text().size();

	if(len == 0)
		return 0;

	long durMS = sub.get_duration().totalmsecs;

	if(durMS == 0)
		return 0;

	double dsX = len * 1000 / durMS;
	int ds = (int)(round(dsX*10) / 10);
	return ds;
}


/*
 *
 */
class ErrorsView : public Gtk::TreeView
{
public:


	class Column : public Gtk::TreeModel::ColumnRecord
	{
	public:
		Column()
		{
			add(line);
			add(text);
		}
		
		Gtk::TreeModelColumn<Glib::ustring>	line;
		Gtk::TreeModelColumn<Glib::ustring>	text;
	};


public:

	/*
	 *
	 */
	ErrorsView(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
	:Gtk::TreeView(cobject)
	{
		m_model = Gtk::ListStore::create(m_column);

		set_model(m_model);

		// line
		{
			Gtk::TreeViewColumn* column = NULL;
			Gtk::CellRendererText* renderer = NULL;

			column = manage(new Gtk::TreeViewColumn(_("Line")));
			renderer = manage(new Gtk::CellRendererText);

			column->pack_start(*renderer, false);
			column->add_attribute(renderer->property_text(), m_column.line);

			append_column(*column);
		}
		// error
		{
			Gtk::TreeViewColumn* column = NULL;
			Gtk::CellRendererText* renderer = NULL;

			column = manage(new Gtk::TreeViewColumn(_("Errors")));
			renderer = manage(new Gtk::CellRendererText);

			column->pack_start(*renderer, false);
			column->add_attribute(renderer->property_markup(), m_column.text);

			append_column(*column);

			//column->set_cell_data_func(*renderer, sigc::mem_fun(*this, &ErrorsView::on_cell_data_func));
		}

		set_rules_hint(true);

		get_selection()->signal_changed().connect(
				sigc::mem_fun(*this, &ErrorsView::on_selection_changed));
		show_all();
	}

	/*
	 *
	 */
	void on_cell_data_func(Gtk::CellRenderer *cell, const Gtk::TreeIter &iter)
	{
		Gtk::CellRendererText *text = dynamic_cast<Gtk::CellRendererText*>(cell);

		text->property_markup() = (*iter)[m_column.text];
	}

	/*
	 *
	 */
	void add_message(unsigned int line, const Glib::ustring &text)
	{
		Gtk::TreeIter iter = m_model->append();

		(*iter)[m_column.line] = to_string(line);
		(*iter)[m_column.text] = text;
	}

	/*
	 *
	 */
	void clear()
	{
		m_model->clear();
	}

	/*
	 *
	 */
	void on_selection_changed()
	{
		Gtk::TreeIter iter = get_selection()->get_selected();

		if(!iter)
			return;

		Glib::ustring line = (*iter)[m_column.line];

		unsigned int num = 0;
		if(from_string(line, num))
		{
			Document *doc = DocumentSystem::getInstance().getCurrentDocument();
			if(doc)
			{
				Gtk::TreeIter sub = doc->get_subtitle_model()->find(num);
				if(sub)
					doc->get_subtitle_view()->select_and_set_cursor(sub);
			}
		}
	}
protected:
	Column m_column;
	Glib::RefPtr<Gtk::ListStore> m_model;
};

/*
 *
 */
class DialogCheckErrorsPreferences : public Gtk::Dialog
{
public:
	DialogCheckErrorsPreferences(BaseObjectType *cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
	:Gtk::Dialog(cobject)
	{
		refGlade->get_widget("check-overlapping", m_checkOverlapping);
		refGlade->get_widget("check-too-short-display-time", m_checkTooShortDisplayTime);
		refGlade->get_widget("check-too-long-display-time", m_checkTooLongDisplayTime);
		refGlade->get_widget("check-too-long-line", m_checkTooLongLine);
		refGlade->get_widget("check-gap-between-subtitle", m_checkGabBetweenSubtitle);
		refGlade->get_widget("check-max-line", m_checkMaxLine);

		refGlade->get_widget("spin-min-characters-per-second", m_spinMinCharactersPerSecond);
		refGlade->get_widget("spin-max-characters-per-second", m_spinMaxCharactersPerSecond);
		refGlade->get_widget("spin-min-gap-between-subtitle", m_spinMinGapBetweenSubtitle);
		refGlade->get_widget("spin-min-display", m_spinMinDisplay);
		refGlade->get_widget("spin-max-characters-per-line", m_spinMaxCharactersPerLine);
		refGlade->get_widget("spin-max-line", m_spinMaxLine);

		WidgetToConfig::read_config_and_connect(m_checkOverlapping, "dialog-check-errors", "check-overlapping", "true");
		WidgetToConfig::read_config_and_connect(m_checkTooShortDisplayTime, "dialog-check-errors", "check-too-short-display-time", "true");
		WidgetToConfig::read_config_and_connect(m_checkTooLongDisplayTime, "dialog-check-errors", "check-too-long-display-time", "true");
		WidgetToConfig::read_config_and_connect(m_checkTooLongLine, "dialog-check-errors", "check-too-long-line", "true");
		WidgetToConfig::read_config_and_connect(m_checkGabBetweenSubtitle, "dialog-check-errors", "check-gap-between-subtitle", "true");
		WidgetToConfig::read_config_and_connect(m_checkMaxLine, "dialog-check-errors", "check-max-line", "true");

		WidgetToConfig::read_config_and_connect(m_spinMinCharactersPerSecond, "timing", "min-characters-per-second", "5");
		WidgetToConfig::read_config_and_connect(m_spinMaxCharactersPerSecond, "timing", "max-characters-per-second", "25");
		WidgetToConfig::read_config_and_connect(m_spinMinGapBetweenSubtitle, "timing", "min-gap-between-subtitle", "100");
		WidgetToConfig::read_config_and_connect(m_spinMinDisplay, "timing", "min-display", "1000");
		WidgetToConfig::read_config_and_connect(m_spinMaxCharactersPerLine, "timing", "max-characters-per-line", "40");
		WidgetToConfig::read_config_and_connect(m_spinMaxLine, "timing", "max-line", "2");
	}
protected:
	Gtk::CheckButton* m_checkOverlapping;
	Gtk::CheckButton* m_checkTooShortDisplayTime;
	Gtk::CheckButton* m_checkTooLongDisplayTime;
	Gtk::CheckButton* m_checkTooLongLine;
	Gtk::CheckButton* m_checkGabBetweenSubtitle;
	Gtk::CheckButton* m_checkMaxLine;

	Gtk::SpinButton* m_spinMinCharactersPerSecond;
	Gtk::SpinButton* m_spinMaxCharactersPerSecond;
	Gtk::SpinButton* m_spinMinGapBetweenSubtitle;
	Gtk::SpinButton* m_spinMinDisplay;
	Gtk::SpinButton* m_spinMaxCharactersPerLine;
	Gtk::SpinButton* m_spinMaxLine;
};

/*
 *
 */
class DialogCheckErrors : public Gtk::Window
{
public:
	DialogCheckErrors(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
	:Gtk::Window(cobject)
	{
		refGlade->get_widget_derived("treeview-errors", m_treeviewErrors);

		refGlade->get_widget("button-refresh", m_buttonRefresh);
		refGlade->get_widget("button-preferences", m_buttonPreferences);

		m_buttonRefresh->signal_clicked().connect(
				sigc::mem_fun(*this, &DialogCheckErrors::check));

		m_buttonPreferences->signal_clicked().connect(
				sigc::mem_fun(*this, &DialogCheckErrors::on_button_preferences));

		check();

		show();
	}

	void on_button_preferences()
	{
		DialogCheckErrorsPreferences *dialog = NULL;

		Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(get_share_dir("dialog-check-errors-preferences.glade"));

		refXml->get_widget_derived("dialog-check-errors-preferences", dialog);

		dialog->run();

		delete dialog;

		check();
	}

	void check()
	{
		unsigned int m_countOverlapping = 0;
		unsigned int m_countTooShortDisplayTime = 0;
		unsigned int m_countTooLongDisplayTime = 0;
		unsigned int m_countTooLongLine = 0;
		unsigned int m_countGabBetweenSubtitle = 0;
		unsigned int m_countMaxLine = 0;

		bool use_overlapping;
		bool use_too_short_display_time;
		bool use_too_long_display_time;
		bool use_too_long_line;
		bool use_gap_between_subtitle;
		bool use_max_line;

		//
		Config &cfg = Config::getInstance();

		cfg.get_value_bool("dialog-check-errors", "check-overlapping", use_overlapping, true);
		cfg.get_value_bool("dialog-check-errors", "check-too-short-display-time", use_too_short_display_time, true);
		cfg.get_value_bool("dialog-check-errors", "check-too-long-display-time", use_too_long_display_time, true);
		cfg.get_value_bool("dialog-check-errors", "check-too-long-line", use_too_long_line, true);
		cfg.get_value_bool("dialog-check-errors", "check-gap-between-subtitle", use_gap_between_subtitle, true);
		cfg.get_value_bool("dialog-check-errors", "check-max-line", use_max_line, true);
		//
		m_treeviewErrors->clear();

		Document *doc = DocumentSystem::getInstance().getCurrentDocument();

		g_return_if_fail(doc);

		CheckErrors checkError;

		Gtk::TreeNodeChildren rows = doc->get_subtitle_model()->children();

		Glib::ustring msg;

		for(Gtk::TreeIter it = rows.begin(); it; ++it)
		{
			SubtitleModifier subtitle(it);

			if(use_overlapping && checkError.check_overlapping(subtitle, msg))
			{
				m_treeviewErrors->add_message(subtitle.get_num(), msg);
				++m_countOverlapping;
			}

			if(use_too_short_display_time && checkError.check_short_display_time(subtitle, msg))
			{
				m_treeviewErrors->add_message(subtitle.get_num(), msg);
				++m_countTooShortDisplayTime;
			}

			if(use_too_long_display_time && checkError.check_long_display_time(subtitle, msg))
			{
				m_treeviewErrors->add_message(subtitle.get_num(), msg);
				++m_countTooLongDisplayTime;
			}

			if(use_too_long_line && checkError.check_long_line_text(subtitle, msg))
			{
				m_treeviewErrors->add_message(subtitle.get_num(), msg);
				++m_countTooLongLine;
			}

			if(use_too_long_line && checkError.check_long_line_translation(subtitle, msg))
			{
				m_treeviewErrors->add_message(subtitle.get_num(), msg);
				++m_countTooLongLine;
			}
			
			if(use_gap_between_subtitle && checkError.check_minimum_gap_between_subtitle(subtitle, msg))
			{
				m_treeviewErrors->add_message(subtitle.get_num(), msg);
				++m_countGabBetweenSubtitle;
			}

			if(use_max_line && checkError.check_max_line_text(subtitle, msg))
			{
				m_treeviewErrors->add_message(subtitle.get_num(), msg);
				++m_countMaxLine;
			}

			if(use_max_line && checkError.check_max_line_translation(subtitle, msg))
			{
				m_treeviewErrors->add_message(subtitle.get_num(), msg);
				++m_countMaxLine;
			}

			//if(checkError.check_minimum_display_subtitle(subtitle, msg))
			//	m_treeviewErrors->add_message(subtitle.get_num(), _("Too short display"));		
		}
		/*
		// display errors
		m_checkOverlapping->set_label(build_message(_("Overlapping <i>(%i errors)</i>"), m_countOverlapping));
		m_checkTooShortDisplayTime->set_label(build_message(_("Too short display time (%i errors)"), m_countTooShortDisplayTime));
		m_checkTooLongDisplayTime->set_label(build_message(_("Too long display time (%i errors)"), m_countTooLongDisplayTime));
		m_checkTooLongLine->set_label(build_message(_("Too long line (%i errors)"), m_countTooLongLine));
		m_checkGabBetweenSubtitle->set_label(build_message(_("Gab between subtitle (%i errors)"), m_countGabBetweenSubtitle));
		*/
	}
protected:
	Gtk::Button* m_buttonRefresh;
	Gtk::Button* m_buttonPreferences;
	ErrorsView* m_treeviewErrors;
};


void createDialogCheckErrors()
{
	DialogCheckErrors *dialog = NULL;

	Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(get_share_dir("dialog-check-errors.glade"));

	refXml->get_widget_derived("dialog-check-errors", dialog);

	dialog->show();
	//dialog->execute();
	//delete dialog;
}

